/*
 * File: main.c
 * Author: Holger Klawitter
 * Datum: 23.3.2000
 *
 */

#include <Pilot.h>

#include "main.h"
#include "forms.rch"

Boolean finished = false;

typedef struct {
	UInt32 syncTime;
	UInt16 state;
} Prefs;

/***********************************************************************
 * Event Handler for the about form
 ***********************************************************************/

#ifdef DEBUG

void DebuggingInfo() {
	Prefs        prefs;
	DateTimeType dateTime;
	Char         tmp[10];

	if( !PrefGetAppPreferencesV10( AppID, 1, &prefs, sizeof(Prefs) ) ) {
		prefs.state = syncNoPrefs;
		prefs.syncTime = 0x10203040;
	}
	
	StrIToA( tmp, prefs.state );
	WinDrawChars( tmp, StrLen(tmp), 20, 120 );
	
	TimSecondsToDateTime( prefs.syncTime, &dateTime );
	StrIToH( tmp, prefs.syncTime );
	WinDrawChars( tmp, StrLen(tmp), 80, 108 );
	
	StrIToA( tmp, dateTime.hour );
	WinDrawChars( tmp, StrLen(tmp), 20, 108 );
	StrIToA( tmp, dateTime.minute );
	WinDrawChars( tmp, StrLen(tmp), 32, 108 );
	StrIToA( tmp, dateTime.second );
	WinDrawChars( tmp, StrLen(tmp), 44, 108 );
}

#else
# define DebuggingInfo()
#endif

Boolean AboutFormHandleEvent( EventPtr eptr )
{
  switch( eptr->eType ) {
  case ctlSelectEvent:
    switch( eptr->data.ctlSelect.controlID ) {
    case AboutFormOk:
			finished = true;
      return true;
    }
    break;
  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
		DebuggingInfo();
    return true;
  default:
  }

  return false;
}

/***********************************************************************
 * MAIN PROGRAM STUFF
 ***********************************************************************/

//#define Debug(x) SetState(x,TimGetSeconds())
#define Debug(x)

void SetState( UInt16 state, UInt32 time ) {
	Prefs prefs;

	prefs.syncTime = time;
	prefs.state = state;
	PrefSetAppPreferencesV10( AppID, 1, &prefs, sizeof(Prefs) );
}

void SetTimeFromDatabase() {
	DmOpenRef db;
	UInt32    diff = 0x00000000;
	LocalID   id;
	UInt16    card;
	Err       err;
	Prefs     prefs;
	MemHandle handle;
	VoidPtr   data;

	// Get the time of the sync which received the program.
	if( !PrefGetAppPreferencesV10( AppID, 1, &prefs, sizeof(Prefs) ) ) {
		return;
	}

	// Only proper sync data should be used.
	if( prefs.state != syncOk ) {
		return;
	}

	// Get the (empty) database.
	// If this DB does not exist, do not set the time.
	db = DmOpenDatabaseByTypeCreator( DBType, AppID, dmModeReadOnly );
	if( !db ) {	return;	}

	err = DmOpenDatabaseInfo( db, &id, NULL, NULL, &card, NULL );
	if( err ) {	DmCloseDatabase( db ); return; }

	// We assume that the DB has been created at the same time as the
	// last sync has taken place.
	DmDatabaseInfo( card, id, NULL, NULL, NULL, NULL, NULL, NULL,
									NULL, NULL, NULL, NULL, NULL );
	handle = DmQueryRecord( db, 0 );
	if( handle ) {
		data = MemHandleLock( handle );
		MemMove( &diff, data, 4 );
		MemHandleUnlock( handle );
		TimSetSeconds( TimGetSeconds()+diff );
	}

	// This database is not useful any more.
	DmCloseDatabase( db );
	DmDeleteDatabase( card, id );

	// Show the time from DB (for debugging)
	SetState(syncDone,diff);
}

//
// Sets the syncing time into the prefs and schedules an alarm for the
// next second (meaning just ASAP).
//
void InitiateAlarm() {
	DmOpenRef db;
	LocalID   id;
	UInt16    card;
	Err       err;

	// make sure that the preferences are being set
	// mark as unclear
	SetState(syncUnclear,TimGetSeconds());

	// Get the current Program
	db = DmOpenDatabaseByTypeCreator( 'appl', AppID, dmModeReadOnly );
	if( !db ) {	return; }

	// Gets the Programs id and card
	err = DmOpenDatabaseInfo( db, &id, NULL, NULL, &card, NULL );
	if( err ) {
		DmCloseDatabase( db );
		return;
	}

	// Set the alarm
	AlmSetAlarm( card, id, NULL, TimGetSeconds()+10, 0 );

	// Save the current sync time
	SetState(syncOk,TimGetSeconds());
}

/***********************************************************************
 * The master Event handler stuff
 ***********************************************************************/
static Boolean ApplicationHandleEvent( EventPtr eptr )
{
  FormPtr frm;
  Int16 formId;
  Boolean handled = false;

  if( eptr->eType == frmLoadEvent ) {
    formId = eptr->data.frmLoad.formID;

    frm = FrmInitForm( formId );
    FrmSetActiveForm( frm );

    switch( formId ) {
    case AboutForm:
      FrmSetEventHandler( frm, (AboutFormHandleEvent) );
      break;
    }
    handled = true;
  }  
  return handled;
}

static void EventLoop()
{
  EventType e;
	UInt16    err;

  do {
    EvtGetEvent( &e, evtWaitForever );
    if( SysHandleEvent( &e ) ) continue;
		if( MenuHandleEvent( NULL, &e, &err ) ) continue;
		if( ApplicationHandleEvent( &e ) ) continue;
		FrmDispatchEvent( &e );
  } while( e.eType != appStopEvent && !finished );
}

/***********************************************************************
 * The Main loop
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, VoidPtr cmdPBP, UInt16 launchFlags)
{
  switch( cmd ) {

	case sysAppLaunchCmdNormalLaunch:
    FrmGotoForm( AboutForm );
    EventLoop();
		break;

	case sysAppLaunchCmdSyncNotify:
		InitiateAlarm();
		break;

	case sysAppLaunchCmdDisplayAlarm:
		SetTimeFromDatabase();
		break;
  } 

  return 0;
}
